package Xconfig::glx;




use common;

my $xgl_config = '/etc/sysconfig/xgl';
my $compiz_config = '/etc/sysconfig/compiz';

sub parse_glxinfo {
    my (%options) = @_;

    my $glx_info = {};
    my $extensions_section;

    my @glxinfo = $options{file} ? cat_($options{file}) : do {
        require run_program;
        my $lib = arch() =~ /x86_64/ ? "lib64" : "lib";
        local $ENV{LD_LIBRARY_PATH} = $options{Mesa} && "/usr/$lib/mesa";
        run_program::rooted_get_stdout($::prefix, '/usr/bin/glxinfo', '2>', '/dev/null');
    };
    foreach (@glxinfo) {
        chomp;
        if (/^direct rendering: Yes/) {
            $glx_info->{capabilities}{direct_rendering} = 1;
        } elsif (my ($type, $value) = /^(.*) string: (.*)$/) {
            $type =~ s/\s/_/g;
            $glx_info->{strings}{$type} = $value;
        } elsif (/^(.*) extensions:/) {
            $extensions_section = $1;
            $extensions_section =~ s/\s/_/g;
        } elsif (/^\s/) {
            if ($extensions_section) {
                push @{$glx_info->{extensions}{$extensions_section}}, grep { $_ } split(/,?\s+/);
            }
        } else {
            undef $extensions_section;
        }
    }
    $glx_info;
}

my @glx_blacklist = qw(nv sis tdfx vmware);

sub get_glx_status {
    my ($o_mesa_file, $o_system_file) = @_;

    my ($driver, $resolution);
    require Xconfig::xfree;
    eval {
        my $raw_X = Xconfig::xfree->read;
        $driver = $raw_X->get_Driver;
        $resolution = $raw_X->get_resolution;
    };
    $resolution ||= {};
    member($driver, @glx_blacklist) and return {};

    my ($mesa_glx_info, $system_glx_info);
    if ($o_mesa_file) {
        $mesa_glx_info = parse_glxinfo(file => $o_mesa_file);
        $system_glx_info = $o_system_file ? parse_glxinfo(file => $o_system_file) : $mesa_glx_info;
    } else {
        $system_glx_info = parse_glxinfo();
        $mesa_glx_info = parse_glxinfo(Mesa => 1);
    }

    my $glx_status = {};

    $glx_status->{context}{Mesa} = $system_glx_info->{strings}{OpenGL_renderer} =~ /Mesa/;
    $glx_status->{context}{MesaDRI} = $system_glx_info->{strings}{OpenGL_renderer} =~ /Mesa DRI/;
    $glx_status->{context}{Xgl} =
      member('GLX_EXT_texture_from_pixmap', @{$mesa_glx_info->{extensions}{GLX}}) && (
          #- avoid nv, but detect i810 and fglrx
          member('GL_ARB_fragment_program', @{$mesa_glx_info->{extensions}{OpenGL}}) ||
            !$glx_status->{context}{Mesa} && member('GLX_SGIX_fbconfig', @{$system_glx_info->{extensions}{GLX}})
      );

    $glx_status->{capabilities}{direct_rendering} =
      $system_glx_info->{capabilities}{direct_rendering} ||
      $glx_status->{context}{Xgl};

    $glx_status->{capabilities}{Xgl} =
      $glx_status->{context}{Xgl} ||
        $glx_status->{capabilities}{direct_rendering} && (
            !$glx_status->{context}{MesaDRI} ||
            member('GLX_MESA_allocate_memory', @{$system_glx_info->{extensions}{GLX}})
          ) && (
            #- Xgl turns black with i810 and 16 bits
            $driver ne 'i810' || $resolution->{Depth} eq '24'
          );

    $glx_status->{capabilities}{AIGLX} =
      $glx_status->{capabilities}{direct_rendering} &&
      member('GLX_EXT_texture_from_pixmap', @{$system_glx_info->{extensions}{client_glx}}) &&
      member('GLX_EXT_texture_from_pixmap', @{$system_glx_info->{extensions}{server_glx}});

    #- no support in configuration tools yet
    # $glx_status->{capabilities}{'3d-desktop'} = $glx_status->{capabilities}{direct_rendering};

    $glx_status;
}

sub detect() {
    my $xgl_config = +{ getVarsFromSh($::prefix . $xgl_config) };
    my $compiz_config = +{ getVarsFromSh($::prefix . $compiz_config) };
    my $glx = { capabilities => get_glx_status()->{capabilities} };
    $glx->{supported} = $glx->{capabilities}{direct_rendering};
    $glx->{compiz} = $compiz_config->{COMPIZ} eq 'yes' || $compiz_config->{config} eq 'auto' && $glx->{supported};
    $glx->{method} =
      $glx->{compiz} ? (
        $glx->{capabilities}{AIGLX} && member($xgl_config->{XGL}, qw(auto no)) ?
          'AIGLX' :
        $glx->{capabilities}{Xgl} && member($xgl_config->{XGL}, qw(auto yes)) ?
          'Xgl' : ''
      ) : '';
    $glx;
}

sub detect_may_install {
    my ($in) = @_;
    $in->do_pkgs->ensure_is_installed('mesa-demos', '/usr/bin/glxinfo') or return;

    my $glx = Xconfig::glx::detect();
    if ($glx->{supported}) {
        my $package = 'task-3ddesktop';
        $in->do_pkgs->is_installed($package) || $in->do_pkgs->ensure_is_installed($package) or return;
    }
    $glx;
}

sub choose_interactive {
    my ($in, $glx) = @_;
    $in->ask_from_({ title => N("3D Desktop effects") }, [
	{ label => N("3D Desktop effects"), title => 1 },
        if_(!$glx->{supported},
            { label => N("Your system does not support 3D desktop effects.") }),
        { type => 'list', val => \$glx->{method}, allow_empty_list => 1,
          list => [ '', grep { $glx->{capabilities}{$_} } qw(AIGLX Xgl 3d-desktop) ],
          format => sub { $_[0] ? N("Use %s", $_[0]) : N("No 3D desktop effects") } },
    ]) or return;
}

sub write {
    my ($glx) = @_;
    addVarsInSh($::prefix . $xgl_config, { XGL => bool2yesno($glx->{method} eq 'Xgl') });
    addVarsInSh($::prefix . $compiz_config, { COMPIZ => bool2yesno(member($glx->{method}, qw(AIGLX Xgl))) });
}

1;
